@@ -1,4 +1,4 @@
-from abc import ABC, abstractmethod
+from abc import ABCMeta, abstractmethod
 from collections.abc import AsyncGenerator, Awaitable, Coroutine
 from typing import Any, Callable, Generic, Optional, TypeVar, Union, overload, runtime_checkable
 
@@ -35,8 +35,10 @@ class JsonBaseSchema(TypedDict):
 
 class PluginJsonBaseSchema(JsonBaseSchema):
     store: NotRequired[Optional[bool]]
-    onSet: NotRequired[Callable[[Any, Any], Awaitable[Union[None, Any]]]]
-    onGet: NotRequired[Callable[[], Awaitable[Union[Any, None]]]]
+    onSet: NotRequired[
+        Union[Callable[[Any, Any], Awaitable[Union[None, Any]]], Callable[[Any, Any], Union[None, Any]]]
+    ]
+    onGet: NotRequired[Union[Callable[[], Awaitable[Union[Any, None]]], Callable[[], Union[Any, None]]]]
 
 
 class PluginJsonSchemaString(PluginJsonBaseSchema):
@@ -87,7 +89,9 @@ class PluginRootSchema(TypedDict):
 
 class JsonSchemaString(JsonBaseSchema):
     type: Literal["string"]
-    format: NotRequired[str]
+    format: NotRequired[
+        Literal["date-time", "date", "time", "email", "uuid", "ipv4", "ipv6", "password", "qrCode", "image"]
+    ]
     minLength: NotRequired[int]
     maxLength: NotRequired[int]
 
@@ -121,7 +125,7 @@ class JsonSchemaButton(TypedDict):
 
 
 class JsonSchemaObjectWithButtons(JsonSchemaObject):
-    buttons: NotRequired[tuple[JsonSchemaButton, NotRequired[JsonSchemaButton]]]
+    buttons: tuple[JsonSchemaButton, NotRequired[JsonSchemaButton]]
 
 
 class JsonSchemaAnyOf(TypedDict):
@@ -317,7 +321,7 @@ CameraPublicProperties = Literal[
 
 
 # camera device
-ObjectClasses = Literal[
+ObjectClass = Literal[
     "person",
     "bicycle",
     "car",
@@ -404,7 +408,7 @@ ObjectClasses = Literal[
 
 class Detection(TypedDict):
     id: NotRequired[str]
-    label: ObjectClasses
+    label: ObjectClass
     confidence: float
     boundingBox: tuple[float, float, float, float]
     inputWidth: int
@@ -631,32 +635,43 @@ PrebufferState = Union[PrebufferStateTrue, PrebufferStateFalse]
 Container = Literal["mpegts", "mp4"]
 
 
-class CameraDelegate(ABC):
-    @abstractmethod
-    async def snapshot_request(self, request_id: str) -> bytes:
-        pass
-
-    @abstractmethod
-    async def prepare_stream(self, request_id: str) -> None:
-        pass
-
-    @abstractmethod
-    async def on_offer(self, request_id: str, sdp: str) -> str:
-        pass
-
-    @abstractmethod
-    async def on_candidates(self, request_id: str, candidates: str) -> None:
-        pass
-
-    @abstractmethod
-    async def reboot(self) -> None:
-        pass
+TalkbackCodec = Literal["PCMA", "PCMU", "opus"]
 
 
-class CameraPrebufferDelegate(ABC):
+class CameraDelegate(metaclass=ABCMeta):
     @abstractmethod
-    async def get_stream_info(self, source_name: str) -> Optional[StreamInfo]:
-        pass
+    async def snapshotRequest(self, request_id: str) -> Union[bytes, None]: ...
+
+    @abstractmethod
+    async def startStream(self, request_id: str) -> None: ...
+
+    @abstractmethod
+    async def stopStream(self, request_id: str) -> None: ...
+
+    @abstractmethod
+    async def onTalkbackSdp(
+        self, request_id: str, source_name: str, sdp: str, codec: TalkbackCodec
+    ) -> None: ...
+
+    @abstractmethod
+    async def onTalkbackRtp(self, request_id: str, source_name: str, rtp: bytes) -> None: ...
+
+    @abstractmethod
+    async def reboot(self) -> None: ...
+
+
+class CameraPrebufferDelegate(metaclass=ABCMeta):
+    @abstractmethod
+    async def getStreamInfo(self, source_name: str) -> Optional[StreamInfo]: ...
+
+
+class CameraControllerOptions(TypedDict):
+    delegate: Optional[CameraDelegate]
+    prebuffer_delegate: Optional[CameraPrebufferDelegate]
+
+
+class CameraDelegateController(Protocol):
+    def configure_controller(self, controller_options: CameraControllerOptions) -> None: ...
 
 
 class CameraSource(Protocol):
@@ -807,9 +822,10 @@ class CameraDevice(Protocol):
     snapshot_source: CameraSource
     recording_source: CameraSource
     detection_source: CameraSource
-    delegate: Optional[CameraDelegate] = None
-    prebuffer_delegate: Optional[CameraPrebufferDelegate] = None
 
+    camera_controller: CameraDelegateController
+
+    def configure_controller(self, controller_options: CameraControllerOptions) -> None: ...
     async def connect(self) -> None: ...
     async def disconnect(self) -> None: ...
     async def reboot(self) -> None: ...
@@ -937,21 +953,27 @@ class BaseCameraConfig(TypedDict):
 
 
 class CameraInputSettings(TypedDict):
-    _id: str
+    # _id: str
     name: str
     roles: list[CameraRoles]
     urls: list[str]
 
 
-class CameraConfigWithSources(BaseCameraConfig):
-    sources: NotRequired[list[CameraInputSettings]]
+class CameraConfigInputSettings(TypedDict):
+    name: str
+    roles: list[CameraRoles]
+    urls: NotRequired[list[str]]
+
+
+class CameraConfigWithoutSources(BaseCameraConfig):
+    sources: list[CameraConfigInputSettings]
 
 
 class CameraConfigWithDelegate(BaseCameraConfig):
     delegate: NotRequired[CameraDelegate]
 
 
-CameraConfig = Union[CameraConfigWithSources, CameraConfigWithDelegate]
+CameraConfig = CameraConfigWithoutSources
 
 
 # events and callbacks
@@ -1030,6 +1052,8 @@ P = TypeVar("P")
 
 @runtime_checkable
 class CameraStorage(Protocol):
+    values: dict[str, Any]
+
     async def initialize_storage(self) -> None: ...
     @overload
     def get_value(self, path: str) -> Union[Awaitable[P], P, None]: ...
@@ -1144,10 +1168,24 @@ PluginConfig = dict[str, Any]
 JSONValue = Union[str, int, float, bool, dict[str, Any], list[Any]]
 
 
-class BasePlugin(ABC):
+class ToastMessage(TypedDict):
+    type: Literal["info", "success", "warning", "error"]
+    message: str
+
+
+class FormSubmitSchema(TypedDict):
+    config: JsonSchemaObjectWithButtons
+
+
+class FormSubmitResponse(TypedDict):
+    toast: NotRequired[ToastMessage]
+    schema: NotRequired[FormSubmitSchema]
+
+
+class BasePlugin(metaclass=ABCMeta):
     @abstractmethod
     def __init__(self, logger: PluginLogger, api_instance: PluginAPI) -> None: ...
     @abstractmethod
-    async def onFormSubmit(self, action_id: str, payload: Any) -> dict[Literal["toastMessage"], str]: ...
+    async def onFormSubmit(self, action_id: str, payload: Any) -> Union[FormSubmitResponse, None]: ...
     @abstractmethod
     def configure_cameras(self, cameras: list[CameraDevice]) -> None: ...
